@@ -8,7 +8,14 @@ set -e
     git submodule update
 }
 
-LIBTOOLIZE=${LIBTOOLIZE:-libtoolize}
+case "$(uname)" in
+    Darwin)
+        LIBTOOLIZE=${LIBTOOLIZE:-glibtoolize}
+        ;;
+    *)
+        LIBTOOLIZE=${LIBTOOLIZE:-libtoolize}
+        ;;
+esac
 AUTORECONF=${AUTORECONF:-autoreconf}
 ACLOCAL=${ACLOCAL:-aclocal}
 AUTOCONF=${AUTOCONF:-autoconf}
