@@ -67,6 +67,8 @@ struct cmd_node {
 
 	const char *token;	/**< Token to enter this cnode */
 	const char *doc;	/**< Documentation string */
+	int privileged;		/**< Privileged command? */
+	int hidden;		/**< Hidden command? */
 
 	/**
 	 * Function validating entry in this node. Can be @c NULL.
@@ -97,7 +99,37 @@ commands_root(void)
 }
 
 /**
- * Create a new node.
+ * Make a node accessible only to privileged users.
+ *
+ * @param node node to change privileges
+ * @return the modified node
+ *
+ * The node is modified. It is returned to ease chaining.
+ */
+struct cmd_node*
+commands_privileged(struct cmd_node *node)
+{
+	if (node) node->privileged = 1;
+	return node;
+}
+
+/**
+ * Hide a node from help or completion.
+ *
+ * @param node node to hide
+ * @return the modified node
+ *
+ * The node is modified. It is returned to ease chaining.
+ */
+struct cmd_node*
+commands_hidden(struct cmd_node *node)
+{
+	if (node) node->hidden = 1;
+	return node;
+}
+
+/**
+ * Create a new node acessible by any user.
  *
  * @param root  The node we want to attach this node.
  * @param token Token to enter this node. Or @c NULL if no token is needed.
@@ -288,6 +320,7 @@ struct candidate_word {
 	TAILQ_ENTRY(candidate_word) next;
 	const char *word;
 	const char *doc;
+	int hidden;
 };
 
 /**
@@ -300,15 +333,17 @@ struct candidate_word {
  * @param argv    Array of arguments.
  * @param word    Completed word. Or NULL when no completion is required.
  * @param all     When completing, display possible completions even if only one choice is possible.
+ * @param priv    Is the current user privileged?
  * @return 0 on success, -1 otherwise.
  */
 static int
 _commands_execute(struct lldpctl_conn_t *conn, struct writer *w,
     struct cmd_node *root, int argc, const char **argv,
-    char **word, int all)
+    char **word, int all, int priv)
 {
 	int n, rc = 0, completion = (word != NULL);
 	int help = 0;		/* Are we asking for help? */
+	int complete = 0;	/* Are we asking for possible completions? */
 	struct cmd_env env = {
 		.elements = TAILQ_HEAD_INITIALIZER(env.elements),
 		.stack = TAILQ_HEAD_INITIALIZER(env.stack),
@@ -322,16 +357,23 @@ _commands_execute(struct lldpctl_conn_t *conn, struct writer *w,
 			log_debug("lldpctl", "argument %02d: `%s`", n, argv[n]);
 	if (completion) *word = NULL;
 
+#define CAN_EXECUTE(candidate) \
+	((!candidate->privileged || priv || complete) && \
+	    (!candidate->validate ||			\
+		candidate->validate(&env, candidate->arg) == 1))
+
 	/* When completion is in progress, we use the same algorithm than for
 	 * execution until we reach the cursor position. */
 	struct cmd_node *current = NULL;
 	while ((current = cmdenv_top(&env))) {
-		if (!completion)
+		if (!completion) {
 			help = !!cmdenv_get(&env, "help"); /* Are we asking for help? */
+			complete = !!cmdenv_get(&env, "complete"); /* Or completion? */
+		}
 
 		struct cmd_node *candidate, *best = NULL;
 		const char *token = (env.argp < env.argc) ? env.argv[env.argp] :
-		    (env.argp == env.argc && !help) ? NEWLINE : NULL;
+		    (env.argp == env.argc && !help && !complete) ? NEWLINE : NULL;
 		if (token == NULL ||
 		    (completion && env.argp == env.argc - 1))
 			goto end;
@@ -340,31 +382,29 @@ _commands_execute(struct lldpctl_conn_t *conn, struct writer *w,
 			    env.argp, token);
 		TAILQ_FOREACH(candidate, &current->subentries, next) {
 			if (candidate->token &&
-			    !strncmp(candidate->token, token, strlen(token))) {
-				if (!candidate->validate ||
-				    candidate->validate(&env, candidate->arg) == 1) {
-					if (candidate->token &&
-					    !strcmp(candidate->token, token)) {
-						/* Exact match */
-						best = candidate;
-						break;
-					}
-					if (!best) best = candidate;
-					else {
-						if (!completion)
-							log_warnx("lldpctl", "ambiguous token: %s (%s or %s)",
-							    token, candidate->token, best->token);
-						rc = -1;
-						goto end;
-					}
+			    !strncmp(candidate->token, token, strlen(token)) &&
+			    CAN_EXECUTE(candidate)) {
+				if (candidate->token &&
+				    !strcmp(candidate->token, token)) {
+					/* Exact match */
+					best = candidate;
+					break;
+				}
+				if (!best) best = candidate;
+				else {
+					if (!completion)
+						log_warnx("lldpctl", "ambiguous token: %s (%s or %s)",
+						    token, candidate->token, best->token);
+					rc = -1;
+					goto end;
 				}
 			}
 		}
 		if (!best) {
 			/* Take first that validate */
 			TAILQ_FOREACH(candidate, &current->subentries, next) {
-				if (!candidate->token && (!candidate->validate ||
-					candidate->validate(&env, candidate->arg) == 1)) {
+				if (!candidate->token &&
+				    CAN_EXECUTE(candidate)) {
 					best = candidate;
 					break;
 				}
@@ -388,30 +428,30 @@ _commands_execute(struct lldpctl_conn_t *conn, struct writer *w,
 		env.argp++;
 	}
 end:
-	if (!completion && !help) {
+	if (!completion && !help && !complete) {
 		if (rc == 0 && env.argp != env.argc + 1) {
 			log_warnx("lldpctl", "incomplete command");
 			rc = -1;
 		}
-	} else if (rc == 0 && (env.argp == env.argc - 1 || help)) {
+	} else if (rc == 0 && (env.argp == env.argc - 1 || help || complete)) {
 		/* We need to complete. Let's build the list of candidate words. */
 		struct cmd_node *candidate = NULL;
-		int maxl = 10;			    /* Max length of a word */
+		size_t maxl = 10;		    /* Max length of a word */
 		TAILQ_HEAD(, candidate_word) words; /* List of subnodes */
 		TAILQ_INIT(&words);
 		current = cmdenv_top(&env);
 		if (!TAILQ_EMPTY(&current->subentries)) {
 			TAILQ_FOREACH(candidate, &current->subentries, next) {
-				if ((!candidate->token || help ||
+				if ((!candidate->token || help || complete ||
 					!strncmp(env.argv[env.argc - 1], candidate->token,
 					    strlen(env.argv[env.argc -1 ]))) &&
-				    (!candidate->validate ||
-					candidate->validate(&env, candidate->arg) == 1)) {
+				    CAN_EXECUTE(candidate)) {
 					struct candidate_word *cword =
 					    malloc(sizeof(struct candidate_word));
 					if (!cword) break;
 					cword->word = candidate->token;
 					cword->doc = candidate->doc;
+					cword->hidden = candidate->hidden;
 					if (cword->word && strlen(cword->word) > maxl)
 						maxl = strlen(cword->word);
 					TAILQ_INSERT_TAIL(&words, cword, next);
@@ -420,14 +460,14 @@ end:
 		}
 		if (!TAILQ_EMPTY(&words)) {
 			/* Search if there is a common prefix, then return it. */
-			int c = 0;
 			char prefix[maxl + 2]; /* Extra space may be added at the end */
 			struct candidate_word *cword, *cword_next;
 			memset(prefix, 0, maxl+2);
-			for (int n = 0; n < maxl; n++) {
-				c = 1; /* Set to 0 to exit outer loop */
+			for (size_t n = 0; n < maxl; n++) {
+				int c = 1; /* Set to 0 to exit outer loop */
 				TAILQ_FOREACH(cword, &words, next) {
 					c = 0;
+					if (cword->hidden) continue;
 					if (cword->word == NULL) break;
 					if (!strcmp(cword->word, NEWLINE)) break;
 					if (strlen(cword->word) == n) break;
@@ -442,7 +482,7 @@ end:
 			}
 			/* If the prefix is complete, add a space, otherwise,
 			 * just return it as is. */
-			if (!all && !help && strcmp(prefix, NEWLINE) &&
+			if (!all && !help && !complete && strcmp(prefix, NEWLINE) &&
 			    strlen(prefix) > 0 &&
 			    strlen(env.argv[env.argc-1]) < strlen(prefix)) {
 				TAILQ_FOREACH(cword, &words, next) {
@@ -454,16 +494,27 @@ end:
 				*word = strdup(prefix);
 			} else {
 				/* No common prefix, print possible completions */
-				fprintf(stderr, "\n-- \033[1;34m%s\033[0m\n",
-				    current->doc ? current->doc : "Help");
+				if (!complete)
+					fprintf(stderr, "\n-- \033[1;34m%s\033[0m\n",
+					    current->doc ? current->doc : "Help");
 				TAILQ_FOREACH(cword, &words, next) {
+					if (cword->hidden) continue;
+
 					char fmt[100];
-					snprintf(fmt, sizeof(fmt),
-					    "%s%%%ds%s  %%s\n",
-					    "\033[1;30m", maxl, "\033[0m");
-					fprintf(stderr, fmt,
-					    cword->word ? cword->word : "WORD",
-					    cword->doc ?  cword->doc  : "...");
+					if (!complete) {
+						snprintf(fmt, sizeof(fmt),
+						    "%s%%%ds%s  %%s\n",
+						    "\033[1;30m", (int)maxl, "\033[0m");
+						fprintf(stderr, fmt,
+						    cword->word ? cword->word : "WORD",
+						    cword->doc ?  cword->doc  : "...");
+					} else {
+						if (!cword->word || !strcmp(cword->word, NEWLINE))
+							continue;
+						fprintf(stdout, "%s %s\n",
+						    cword->word ? cword->word : "WORD",
+						    cword->doc ?  cword->doc  : "...");
+					}
 				}
 			}
 			for (cword = TAILQ_FIRST(&words); cword != NULL;
@@ -483,11 +534,11 @@ end:
  */
 char *
 commands_complete(struct cmd_node *root, int argc, const char **argv,
-    int all)
+    int all, int privileged)
 {
 	char *word = NULL;
 	if (_commands_execute(NULL, NULL, root, argc, argv,
-		&word, all) == 0)
+		&word, all, privileged) == 0)
 		return word;
 	return NULL;
 }
@@ -497,9 +548,9 @@ commands_complete(struct cmd_node *root, int argc, const char **argv,
  */
 int
 commands_execute(struct lldpctl_conn_t *conn, struct writer *w,
-    struct cmd_node *root, int argc, const char **argv)
+    struct cmd_node *root, int argc, const char **argv, int privileged)
 {
-	return _commands_execute(conn, w, root, argc, argv, NULL, 0);
+	return _commands_execute(conn, w, root, argc, argv, NULL, 0, privileged);
 }
 
 /**
@@ -676,3 +727,20 @@ cmd_restrict_ports(struct cmd_node *root)
 		"Restrict configuration to the specified ports (comma-separated list)",
 		NULL, cmd_store_env_value_and_pop2, "ports");
 }
+
+/**
+ * Restrict the command to specific protocol
+ */
+void
+cmd_restrict_protocol(struct cmd_node *root)
+{
+	/* Restrict to some ports. */
+	commands_new(
+		commands_new(root,
+		    "protocol",
+		    "Restrict to specific protocol",
+		    cmd_check_no_env, NULL, "protocol"),
+		NULL,
+		"Restrict display to the specified protocol",
+		NULL, cmd_store_env_value_and_pop2, "protocol");
+}
