@@ -504,12 +504,17 @@ display_age(time_t lastchange)
 
 void
 display_interface(lldpctl_conn_t *conn, struct writer *w, int hidden,
-    lldpctl_atom_t *iface, lldpctl_atom_t *neighbor, int details)
+    lldpctl_atom_t *iface, lldpctl_atom_t *neighbor, int details, int protocol)
 {
 	if (!hidden &&
 	    lldpctl_atom_get_int(neighbor, lldpctl_k_port_hidden))
 		return;
 
+	/* user might have specified protocol to filter on display */
+	if ((protocol != LLDPD_MODE_MAX) &&
+	    (protocol != lldpctl_atom_get_int(neighbor, lldpctl_k_port_protocol)))
+	    return;
+
 	tag_start(w, "interface", "Interface");
 	tag_attr(w, "name", "",
 	    lldpctl_atom_get_str(iface, lldpctl_k_interface_name));
@@ -549,6 +554,26 @@ display_interfaces(lldpctl_conn_t *conn, struct writer *w,
     int hidden, int details)
 {
 	lldpctl_atom_t *iface;
+	int protocol = LLDPD_MODE_MAX;
+	const char *proto_str;
+
+	/* user might have specified protocol to filter display results */
+	proto_str = cmdenv_get(env, "protocol");
+
+	if (proto_str) {
+		log_debug("display", "filter protocol: %s ", proto_str);
+
+		protocol = 0;
+		for (lldpctl_map_t *protocol_map =
+			 lldpctl_key_get_map(lldpctl_k_port_protocol);
+		     protocol_map->string;
+		     protocol_map++) {
+			if (!strcasecmp(proto_str, protocol_map->string)) {
+				protocol = protocol_map->value;
+				break;
+			}
+		}
+	}
 
 	tag_start(w, "lldp", "LLDP neighbors");
 	while ((iface = cmd_iterate_on_interfaces(conn, env))) {
@@ -558,7 +583,7 @@ display_interfaces(lldpctl_conn_t *conn, struct writer *w,
 		port      = lldpctl_get_port(iface);
 		neighbors = lldpctl_atom_get(port, lldpctl_k_port_neighbors);
 		lldpctl_atom_foreach(neighbors, neighbor) {
-			display_interface(conn, w, hidden, iface, neighbor, details);
+			display_interface(conn, w, hidden, iface, neighbor, details, protocol);
 		}
 		lldpctl_atom_dec_ref(neighbors);
 		lldpctl_atom_dec_ref(port);
@@ -731,6 +756,9 @@ display_configuration(lldpctl_conn_t *conn, struct writer *w)
 	tag_datatag(w, "ifdescr-update", "Update interface descriptions",
 	    lldpctl_atom_get_int(configuration, lldpctl_k_config_ifdescr_update)?
 	    "yes":"no");
+	tag_datatag(w, "iface-promisc", "Promiscuous mode on managed interfaces",
+	    lldpctl_atom_get_int(configuration, lldpctl_k_config_iface_promisc)?
+	    "yes":"no");
 	tag_datatag(w, "lldpmed-no-inventory", "Disable LLDP-MED inventory",
 	    (lldpctl_atom_get_int(configuration, lldpctl_k_config_lldpmed_noinventory) == 0)?
 	    "no":"yes");
