@@ -29,6 +29,7 @@
 #include <arpa/inet.h>
 #include <libgen.h>
 #include <dirent.h>
+#include <signal.h>
 #include <sys/queue.h>
 
 #include "client.h"
@@ -66,10 +67,10 @@ usage()
 	fprintf(stderr, "\n");
 
 	fprintf(stderr, "-d          Enable more debugging information.\n");
-	fprintf(stderr, "-u          Specify the Unix-domain socket used for communication with lldpd(8).\n");
-	fprintf(stderr, "-f format   Choose output format (plain, keyvalue or xml).\n");
+	fprintf(stderr, "-u socket   Specify the Unix-domain socket used for communication with lldpd(8).\n");
+	fprintf(stderr, "-f format   Choose output format (plain, keyvalue, json or xml).\n");
 	if (!is_lldpctl(NULL))
-		fprintf(stderr, "-c          Read the provided configuration file.\n");
+		fprintf(stderr, "-c conf     Read the provided configuration file.\n");
 
 	fprintf(stderr, "\n");
 
@@ -156,10 +157,10 @@ cmd_pause_resume(lldpctl_conn_t *conn, int pause)
 		return 0;
 	}
 	if (lldpctl_atom_get_int(config, lldpctl_k_config_paused) == pause) {
-		log_info("lldpctl", "lldpd is already %s",
+		log_debug("lldpctl", "lldpd is already %s",
 		    pause?"paused":"resumed");
 		lldpctl_atom_dec_ref(config);
-		return 0;
+		return 1;
 	}
 	if (lldpctl_atom_set_int(config,
 		lldpctl_k_config_paused, pause) == NULL) {
@@ -205,7 +206,7 @@ _cmd_complete(int all)
 	if (tokenize_line(line, &argc, &argv) != 0)
 		goto end;
 
-	char *compl = commands_complete(root, argc, (const char **)argv, all);
+	char *compl = commands_complete(root, argc, (const char **)argv, all, is_privileged());
 	if (compl && strlen(argv[argc-1]) < strlen(compl)) {
 		if (rl_insert_text(compl + strlen(argv[argc-1])) < 0) {
 			free(compl);
@@ -270,14 +271,17 @@ cmd_exec(lldpctl_conn_t *conn, const char *fmt, int argc, const char **argv)
 #ifdef USE_XML
 	else if (strcmp(fmt, "xml")      == 0) w = xml_init(stdout);
 #endif
-#ifdef USE_JSON
-	else if (strcmp(fmt, "json")     == 0) w = json_init(stdout);
+#ifdef USE_JANSSON
+	else if (strcmp(fmt, "json")     == 0) w = jansson_init(stdout);
+#endif
+#ifdef USE_JSONC
+	else if (strcmp(fmt, "json")     == 0) w = jsonc_init(stdout);
 #endif
 	else w = txt_init(stdout);
 
 	/* Execute command */
 	int rc = commands_execute(conn, w,
-	    root, argc, argv);
+	    root, argc, argv, is_privileged());
 	if (rc != 0) {
 		log_info("lldpctl", "an error occurred while executing last command");
 		w->finish(w);
@@ -324,14 +328,14 @@ register_commands()
 	root = commands_root();
 	register_commands_show(root);
 	register_commands_watch(root);
-	if (is_privileged()) {
-		commands_new(
-			commands_new(root, "update", "Update information and send LLDPU on all ports",
-			    NULL, NULL, NULL),
-			NEWLINE, "Update information and send LLDPU on all ports",
-			NULL, cmd_update, NULL);
-		register_commands_configure(root);
-	}
+	commands_privileged(commands_new(
+		commands_new(root, "update", "Update information and send LLDPU on all ports",
+		    NULL, NULL, NULL),
+		NEWLINE, "Update information and send LLDPU on all ports",
+		NULL, cmd_update, NULL));
+	register_commands_configure(root);
+	commands_hidden(commands_new(root, "complete", "Get possible completions from a given command",
+		NULL, cmd_store_env_and_pop, "complete"));
 	commands_new(root, "help", "Get help on a possible command",
 	    NULL, cmd_store_env_and_pop, "help");
 	commands_new(
@@ -426,6 +430,8 @@ main(int argc, char *argv[])
 
 	ctlname = lldpctl_get_default_transport();
 
+	signal(SIGHUP, SIG_IGN);
+
 	/* Initialize logging */
 	while ((ch = getopt(argc, argv, options)) != -1) {
 		switch (ch) {
