@@ -40,11 +40,11 @@ interfaces_setup_multicast(struct lldpd *cfg, const char *name,
 			    cfg->g_protocols[i].mac, !remove)) != 0) {
 			errno = rc;
 			if (errno != ENOENT)
-				log_info("interfaces",
-				    "unable to %s %s address to multicast filter for %s",
+				log_debug("interfaces",
+				    "unable to %s %s address to multicast filter for %s (%s)",
 				    (remove)?"delete":"add",
 				    cfg->g_protocols[i].name,
-				    name);
+				    name, strerror(rc));
 		}
 	}
 }
@@ -563,6 +563,18 @@ interfaces_helper_physical(struct lldpd *cfg,
 	}
 }
 
+void
+interfaces_helper_promisc(struct lldpd *cfg,
+    struct lldpd_hardware *hardware)
+{
+	if (!cfg->g_config.c_promisc) return;
+	if (priv_iface_promisc(hardware->h_ifname) != 0) {
+		log_warnx("interfaces",
+		    "unable to enable promiscuous mode for %s",
+		    hardware->h_ifname);
+	}
+}
+
 /**
  * Send the packet using the hardware function. Optionnaly mangle the MAC address.
  *
