@@ -315,7 +315,7 @@ lldpd_reset_timer(struct lldpd *cfg)
 		cksum = frame_checksum(output, output_len, 0);
 		free(output);
 		if (cksum != hardware->h_lport_cksum) {
-			log_info("localchassis",
+			log_debug("localchassis",
 			    "change detected for port %s, resetting its timer",
 			    hardware->h_ifname);
 			hardware->h_lport_cksum = cksum;
@@ -515,15 +515,22 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 		}
 	}
 	/* Do we have room for a new MSAP? */
-	if (!oport && cfg->g_config.c_max_neighbors &&
-	    count > cfg->g_config.c_max_neighbors - 1) {
-		log_info("decode",
+	if (!oport && cfg->g_config.c_max_neighbors) {
+	    if (count == (cfg->g_config.c_max_neighbors - 1)) {
+		log_debug("decode",
+		    "max neighbors %d reached for port %s, "
+		    "dropping any new ones silently",
+		    cfg->g_config.c_max_neighbors,
+		    hardware->h_ifname);
+	    } else if (count > cfg->g_config.c_max_neighbors - 1) {
+		log_debug("decode",
 		    "too many neighbors for port %s, drop this new one",
 		    hardware->h_ifname);
 		lldpd_port_cleanup(port, 1);
 		lldpd_chassis_cleanup(chassis, 1);
 		free(port);
 		return;
+	    }
 	}
 	/* No, but do we already know the system? */
 	if (!oport) {
@@ -1140,7 +1147,7 @@ lldpd_configure(int debug, const char *path, const char *ctlname)
 			    "-c", SYSCONFDIR "/lldpd.conf",
 			    "-c", SYSCONFDIR "/lldpd.d",
 			    "resume",
-			    NULL);
+			    (char *)NULL);
 			log_warn("main", "unable to execute %s", path);
 			log_warnx("main", "configuration is incomplete, lldpd needs to be unpaused");
 		}
@@ -1398,11 +1405,8 @@ lldpd_main(int argc, char *argv[], char *envp[])
 			found = 0;
 			for (i=0; protos[i].mode != 0; i++) {
 				if (ch == protos[i].arg) {
-					if (protos[i].enabled < 3) {
-						found = 1;
-						if (protos[i].enabled++ == 1)
-							break;
-					}
+					found = 1;
+					protos[i].enabled++;
 				}
 			}
 			if (!found)
@@ -1467,6 +1471,9 @@ lldpd_main(int argc, char *argv[], char *envp[])
 	/* Disable SIGPIPE */
 	signal(SIGPIPE, SIG_IGN);
 
+	/* Disable SIGHUP, until handlers are installed */
+	signal(SIGHUP, SIG_IGN);
+
 	/* Configuration with lldpcli */
 	if (lldpcli) {
 		log_debug("main", "invoking lldpcli for configuration");
@@ -1577,13 +1584,37 @@ lldpd_main(int argc, char *argv[], char *envp[])
 
 	log_debug("main", "initialize protocols");
 	cfg->g_protocols = protos;
-	for (i=0; protos[i].mode != 0; i++)
+	for (i=0; protos[i].mode != 0; i++) {
+
+		/* With -ll, disable LLDP */
+		if (protos[i].mode == LLDPD_MODE_LLDP)
+			protos[i].enabled %= 3;
+		/* With -ccc force CDPV2, enable CDPV1 */
+		if (protos[i].mode == LLDPD_MODE_CDPV1 && protos[i].enabled == 3) {
+			protos[i].enabled = 1;
+		}
+		/* With -cc force CDPV1, enable CDPV2 */
+		if (protos[i].mode == LLDPD_MODE_CDPV2 && protos[i].enabled == 2) {
+			protos[i].enabled = 1;
+		}
+
+		/* With -cccc disable CDPV1, enable CDPV2 */
+		if (protos[i].mode == LLDPD_MODE_CDPV1 && protos[i].enabled >= 4) {
+			protos[i].enabled = 0;
+		}
+
+		/* With -cccc disable CDPV1, enable CDPV2; -ccccc will force CDPv2 */
+		if (protos[i].mode == LLDPD_MODE_CDPV2 && protos[i].enabled == 4) {
+			protos[i].enabled = 1;
+		}
+
 		if (protos[i].enabled > 1)
 			log_info("main", "protocol %s enabled and forced", protos[i].name);
 		else if (protos[i].enabled)
 			log_info("main", "protocol %s enabled", protos[i].name);
 		else
 			log_info("main", "protocol %s disabled", protos[i].name);
+	    }
 
 	TAILQ_INIT(&cfg->g_hardware);
 	TAILQ_INIT(&cfg->g_chassis);
