@@ -334,6 +334,8 @@ lldpctl_key_get_map(lldpctl_key_t key)
 		return bond_slave_src_mac_map;
 	case lldpctl_k_config_lldp_portid_type:
 		return lldp_portid_map;
+	case lldpctl_k_port_protocol:
+		return lldpd_protocol_map;
 	default: return empty_map;
 	}
 }
@@ -480,6 +482,8 @@ _lldpctl_atom_get_int_config(lldpctl_atom_t *atom, lldpctl_key_t key)
 		return c->config->c_advertise_version;
 	case lldpctl_k_config_ifdescr_update:
 		return c->config->c_set_ifdescr;
+	case lldpctl_k_config_iface_promisc:
+		return c->config->c_promisc;
 #ifdef ENABLE_LLDPMED
 	case lldpctl_k_config_lldpmed_noinventory:
 		return c->config->c_noinventory;
@@ -515,6 +519,9 @@ _lldpctl_atom_set_int_config(lldpctl_atom_t *atom, lldpctl_key_t key,
 	case lldpctl_k_config_ifdescr_update:
 		config.c_set_ifdescr = c->config->c_set_ifdescr = value;
 		break;
+	case lldpctl_k_config_iface_promisc:
+		config.c_promisc = c->config->c_promisc = value;
+		break;
 #ifdef ENABLE_LLDPMED
 	case lldpctl_k_config_fast_start_enabled:
 		config.c_enable_fast_start =  value?1:2;
